package com.lowagie.test.functional;

import java.io.*;
import java.util.zip.ZipInputStream;
import java.util.jar.JarInputStream;
import java.util.jar.Manifest;
import java.util.*;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

public class TestRunner {
    private Manifest manifest;
    HashMap packages; // map of package names to ArrayList of classes
    Class baseClass;
    ClassLoader loader;

    public TestRunner() throws IOException {
        packages = new HashMap();
        loader = Thread.currentThread().getContextClassLoader();
        try {
            baseClass = loader.loadClass("com.lowagie.test.functional.FunctionalTestSuite");
        } catch (ClassNotFoundException e) {
            throw new IOException("Can't find com.lowagie.test.functional.FunctionalTestSuite class");
        }
        InputStream instream = loader.getResourceAsStream("itest_filelist.info");
        if (instream == null) {
            throw new IOException("Can't find itest_filelist.info");
        } else {
            parseFileList(instream);
        }

    }

    private void parseFileList(InputStream instream) {
        String nextClassName;
        BufferedReader reader = new BufferedReader(new InputStreamReader(instream));
        try {
            while (reader.ready()) {
                String nextLine = reader.readLine();

                // first remove .class from end
                if (nextLine.length() > 0) {
                    int classIndex = nextLine.lastIndexOf(".class");
                    if (classIndex > 0) {
                        nextLine = nextLine.substring(0, nextLine.lastIndexOf(".class"));
                        nextClassName = nextLine.replace('\\', '.');
                        try {
                            Class nextClass = loader.loadClass(nextClassName);
                            if (isTestClass(nextClass)) {
                                addClass(nextClass);
                            }
                        } catch (ClassNotFoundException e) {
                            // skip it if not found
                        }

                    }

                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void addClass(Class newClass) {
        Package newPackage = newClass.getPackage();
        List classList = (List) packages.get(newPackage.getName());
        if (classList == null) {
            classList = new ArrayList();
            packages.put(newPackage.getName(), classList);
        }
        if (!classList.contains(newClass)) {
            classList.add(newClass);
        }
    }

    private boolean isTestClass(Class testClass) {
        return (!baseClass.equals(testClass) && baseClass.isAssignableFrom(testClass));
    }

    public static void testSuite(String qualifiedSuiteName, OutputStream os) throws Exception {
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        Class suiteClass = loader.loadClass(qualifiedSuiteName);
        if (suiteClass == null) { throw new Exception("Can't find suite:"+qualifiedSuiteName); }
        FunctionalTestSuite suite = (FunctionalTestSuite) suiteClass.newInstance();
        if (suite == null) { throw new Exception("Couldn't make suite:"+qualifiedSuiteName); }
        suite.test(os);
    }

    public static void test(String qualifiedSuiteName, String testName, OutputStream os) throws Exception {
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        Class suiteClass = loader.loadClass(qualifiedSuiteName);
        if (suiteClass == null) { throw new Exception("Can't find suite:"+qualifiedSuiteName); }
        FunctionalTestSuite suite = (FunctionalTestSuite) suiteClass.newInstance();
        suite.test(testName, os);
    }

    public static void debug(String qualifiedSuiteName, String testName, OutputStream os) throws Exception {
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        Class suiteClass = loader.loadClass(qualifiedSuiteName);
        if (suiteClass == null) { throw new Exception("Can't find suite:"+qualifiedSuiteName); }
        FunctionalTestSuite suite = (FunctionalTestSuite) suiteClass.newInstance();
        suite.debug(testName, os);
    }


    public List getSuitePackageNames() throws IOException {
        List result = new ArrayList(packages.keySet());
        Collections.sort(result);
        return result;
    }

    public static void testSuiteToFile(String suiteName) {
        File outDir = new File("TestResults");
        outDir.mkdir();
        File outFile = new File(outDir, suiteName+".pdf");
        try {
            FileOutputStream outStream = new FileOutputStream(outFile);
            testSuite(suiteName, outStream);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private class TestSuiteComparator implements Comparator {
        public int compare(Object o1, Object o2) {
            FunctionalTestSuite suite1 = (FunctionalTestSuite) o1;
            FunctionalTestSuite suite2 = (FunctionalTestSuite) o2;
            return suite1.getName().compareTo(suite2.getName());
        }
    }

    /**
     * Gets all of the FunctionalTestSuites in the specified package.
     *
     * @param packageName
     * @return ist of FunctionalTestSuites
     * @throws IOException
     */
    public List getTestSuites(String packageName) throws IOException {
        ArrayList results = new ArrayList();
        ArrayList suiteList = (ArrayList) packages.get(packageName);
        if (suiteList != null) {
            for (Iterator it = suiteList.iterator(); it.hasNext();) {
                Class suiteClass = (Class) it.next();
                try {
                    FunctionalTestSuite newSuite = (FunctionalTestSuite) suiteClass.newInstance();
                    results.add(newSuite);
                } catch (IllegalAccessException e) {
                } catch (InstantiationException e) {
                    e.printStackTrace();
                }
            }
        }
        Collections.sort(results, new TestSuiteComparator());
        return results;
    }

/*
    private static List getPropertyWithPrefix(String prefix) throws IOException {
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        InputStream propStream = loader.getResourceAsStream("itest.properties");
        Properties testProperties = new Properties();
        testProperties.load(propStream);
        Enumeration keys = testProperties.keys();
        ArrayList results = new ArrayList();
        while (keys.hasMoreElements()) {
            String keyName = (String) keys.nextElement();
            if (keyName.startsWith(prefix)) {
                results.add(testProperties.get(keyName));
            }
        }
        return results;
    }
*/

    public static void main(String[] args) {
        try {
            TestRunner runner = new TestRunner();
            if (args.length == 1 && args[0].equalsIgnoreCase("all")) {
                List packages = runner.getSuitePackageNames();
                for (Iterator packIt = packages.iterator(); packIt.hasNext();) {
                    String packageName = (String) packIt.next();
                    System.out.println("Package "+packageName);
                    List testSuites = runner.getTestSuites(packageName);
                    for (Iterator testIt = testSuites.iterator(); testIt.hasNext();) {
                        String suiteName = ((FunctionalTestSuite)testIt.next()).getClass().getName();
                        System.out.println("    "+suiteName);
                        testSuiteToFile(suiteName);
                    }
                }
            } else if (args.length == 1 && args[0].equalsIgnoreCase("list")) {
                List packages = runner.getSuitePackageNames();
                for (Iterator packIt = packages.iterator(); packIt.hasNext();) {
                    String packageName = (String) packIt.next();
                    System.out.println("Package "+packageName);
                    List testSuites = runner.getTestSuites(packageName);
                    for (Iterator testIt = testSuites.iterator(); testIt.hasNext();) {
                        String suiteName = ((FunctionalTestSuite)testIt.next()).getClass().getName();
                        System.out.println("    "+suiteName);
                    }
                }
            } else if (args.length == 1) {
               testSuiteToFile(args[0]);
            }
            else {
                printHelp();
            }

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void printHelp() {
        System.out.println("Syntax is:");
        System.out.println("  TestRunner list       - lists test packages and tests");
        System.out.println("  TestRunner all        - run all tests");
        System.out.println("  TestRunner nameoftest - run specified test");
    }
}
