package com.lowagie.test.functional.pdf;

import com.lowagie.text.*;
import com.lowagie.text.Font;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.*;

import java.io.IOException;
import java.io.OutputStream;
import java.awt.*;

public class VariablePdfPTableBorderTestSuite extends PdfBaseTestSuite {
    private String description = "Tests Variable style borders on com.lowagie.text.Table when rendered to PDF";

    public VariablePdfPTableBorderTestSuite() {
        pageSize = PageSize.LETTER;
    }

    public VariablePdfPTableBorderTestSuite(OutputStream os) throws DocumentException {
        super(PageSize.LETTER);
    }

    public void testBorderRainbow() throws BadElementException, DocumentException, IOException {
        float padding = 0f;
        Rectangle border = new Rectangle(0f, 0f);
        border.setBorderWidthLeft(6f);
        border.setBorderWidthBottom(5f);
        border.setBorderWidthRight(4f);
        border.setBorderWidthTop(2f);
        border.setBorderColorLeft(Color.RED);
        border.setBorderColorBottom(Color.ORANGE);
        border.setBorderColorRight(Color.YELLOW);
        border.setBorderColorTop(Color.GREEN);
        testBorder(padding, border);
    }

    public void testBorderPadding() throws BadElementException, DocumentException, IOException {
        float padding = 1f;
        Rectangle border = new Rectangle(0f, 0f);
        border.setBorderWidthLeft(.1f);
        border.setBorderWidthBottom(1f);
        border.setBorderWidthRight(.1f);
        border.setBorderWidthTop(1f);
        border.setBorderColorLeft(Color.BLACK);
        border.setBorderColorBottom(Color.LIGHT_GRAY);
        border.setBorderColorRight(Color.BLACK);
        border.setBorderColorTop(Color.LIGHT_GRAY);
        testBorder(padding, border);
    }



    public void testBorderSquareEnds() throws BadElementException, DocumentException, IOException {
        float padding = 0f;
        Rectangle border = new Rectangle(0f, 0f);
        border.setBorderWidthBottom(5f);
        border.setBorderWidthLeft(0f);
        border.setBorderWidthTop(0f);
        border.setBorderWidthRight(0f);
        //border.setBorderColorLeft(Color.BLACK);
        border.setBorderColorBottom(Color.LIGHT_GRAY);
        border.setBorderColorRight(Color.BLACK);
        //border.setBorderColorTop(Color.LIGHT_GRAY);
        testBorder(padding, border);
    }

    private void testBorder(float padding, Rectangle border) throws BadElementException, DocumentException, IOException {
        Font tableFont = FontFactory.getFont("Helvetica", 8, Font.BOLD, Color.BLACK);
        BaseFont baseFont = tableFont.getBaseFont();
        PdfContentByte cb = writer.getDirectContentUnder();
        float targetOffset = 0.2f / 2f;  // 1/2 of line width for target
        float leading = tableFont.size() * 1.2f;
        float spacing = 0f;

        makeTestPage(tableFont, border, writer, document, padding);
        addText("\n" + (border.isUseVariableBorders() ? "Variable" : "Normal") + " border, cellspacing=" + spacing + ", cellpadding=" + padding, Element.ALIGN_CENTER);
        float tableTop = document.getPageSize().height() - document.topMargin();
        float tableLeft = document.leftMargin();

        //outside top left corner
        float markerLeft = tableLeft - targetOffset + spacing -
                (border.isUseVariableBorders() ? 0f : border.getBorderWidthLeft() / 2f);
        float markerTop = tableTop + targetOffset - spacing +
                (border.isUseVariableBorders() ? 0f : border.getBorderWidthTop() / 2f);

        float markerX = markerLeft;
        float markerY = markerTop;

        drawTarget(markerX, markerY, 10);

        // outside top right corner
        float markerRight = document.getPageSize().width() - document.rightMargin() + targetOffset - spacing +
                (border.isUseVariableBorders() ? 0f : border.getBorderWidthRight() / 2f);
        drawTarget(markerRight, markerTop, 10);

        float descent = baseFont.getFontDescriptor(BaseFont.DESCENT, 8);
        float ascent = baseFont.getFontDescriptor(BaseFont.ASCENT, 8);
        // inside upper left corner of cell 5
        float tableRow1Height = ascent -
                descent +
                (padding * 2f) + (2f * spacing) +
                2f * border.getBorderWidthTop() + border.getBorderWidthBottom();

        markerX = markerLeft + targetOffset + border.getBorderWidthLeft();
        markerY -= tableRow1Height + targetOffset * 2f;
        drawTarget(markerX, markerY, 10);

        // outside bottom left corner
        float markerBottom = markerY - ascent - 
                ( (3f* leading) -
                baseFont.getFontDescriptor(BaseFont.DESCENT, 8) +
                border.getBorderWidthBottom() +
                2f * padding);
        drawTarget(markerLeft, markerBottom, 10);

        // outside bottom right corner
        drawTarget(markerRight, markerBottom, 10);

        addText("Checkpoints:", Element.ALIGN_LEFT);
        addText("1) Outside all four corners", Element.ALIGN_LEFT);
        addText("2) Inside the upper left corner of cell 5", Element.ALIGN_LEFT);
    }


    private void makeTestPage(Font tableFont, Rectangle borders, PdfWriter writer, Document document,
                              float padding) throws BadElementException, DocumentException, IOException {
        document.newPage();
        PdfPTable table = null;
        table = new PdfPTable(4);
        table.setWidthPercentage(100f);

        float leading = tableFont.size() * 1.2f;

        table.addCell(makeCell("1-Top", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, leading, padding, borders));
        table.addCell(makeCell("2-Middle", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, leading, padding, borders));
        table.addCell(makeCell("3-Bottom", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, leading, padding, borders));
        table.addCell(makeCell("4-Has a y", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, leading, padding, borders));

        table.addCell(makeCell("5-Abcdy", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, leading, padding, borders));
        table.addCell(makeCell("6-Abcdy", Element.ALIGN_MIDDLE, Element.ALIGN_LEFT, tableFont, leading, padding, borders));
        table.addCell(makeCell("7-Abcdy", Element.ALIGN_BOTTOM, Element.ALIGN_LEFT, tableFont, leading, padding, borders));
        table.addCell(makeCell("8-This\nis\na little\ntaller", Element.ALIGN_TOP, Element.ALIGN_LEFT, tableFont, leading, padding, borders));
        document.add(table);

        float rulerTop = document.getPageSize().top();
        addRuler(1f, document.leftMargin() - 10, rulerTop, rulerTop);
    }

    PdfPCell makeCell(String text, int vAlignment, int hAlignment, Font font, float leading, float padding, Rectangle borders) {
        Paragraph p = new Paragraph(text, font);
        p.setLeading(leading);

        PdfPCell cell = new PdfPCell(p);
        cell.setLeading(leading, 0);
        cell.setVerticalAlignment(vAlignment);
        cell.setHorizontalAlignment(hAlignment);
        cell.cloneNonPositionParameters(borders);
        cell.setUseAscender(true);
        cell.setUseDescender(true);
        cell.setPadding(padding);
        cell.setUseBorderPadding(true);
        return cell;
    }


    Cell makeCell(Element[] elements, int vAlignment, int hAlignment, Rectangle borders) {
        Cell cell = new Cell();
        if (elements != null) {
            for (int i = 0; i < elements.length; i++) {
                Element el = elements[i];
                cell.add(el);
            }
        }
        cell.setVerticalAlignment(vAlignment);
        cell.setHorizontalAlignment(hAlignment);
        cell.cloneNonPositionParameters(borders);
        cell.setUseAscender(true);
        cell.setUseDescender(true);
        cell.setUseBorderPadding(true);
        return cell;
    }

}
